%path_follow.m
%image size is (r, c) = (1200, 1600)
%where 1200 is the bottom of the image, 0 is the top.
%left is 0, right is 1600
close all
clear all

%Uncomment next three lines to read real photo
p1=imread('p1.jpg');
p1g=rgb2gray(p1);
path_start_center = 800; %center column (x value) of path at beginning

%Uncomment next three lines to read test image
%p1=imread('testim.jpg');
%p1g=p1;
%path_start_center = 250; %center column (x value) of path at beginning


p1bw=im2bw(p1g); %threshold to binary image
p1d=im2double(p1bw); %convert image to numeric array
imagesc(p1bw);colormap(summer); %show thresholded binary image
hold on; %freeze image so path can be overlaid

slen=50; %length of each step, in pixels
r1=1200; y1=1201-r1; %start at bottom of image = closest to camera = row 1200
c1=path_start_center; x1=c1; %start in middle of path 
theta=pi/2; %initial heading straight up the page = 90 degrees

step = 0; %initialize path
while ((y1<1199)&&(y1>0)&&(x1<1599)&&(x1>0)&&(step<7)); %keep path in bounds
    %Propose endpoint of next step by following current heading.
    %This will be corrected later by weighted average of pixel values.
    %(x1,y1) = starting point of current step.
    %(x2,y2) = proposed endpoint of current step.
    %(x3,y3) = corrected endpoint of current step.
    %theta = heading angle of current step (relative to x axis, + CCW).
    %theta_t = angle, relative to x-axis,
    %          of a line perpendicular to heading that passes through(x2, y2).
    step=step+1; %move along path
    y2=y1+floor(slen*sin(theta)); %proposed endpoint of step
    x2=x1+floor(slen*cos(theta));
    theta_t=theta-pi/2; %angle of transverse line to current heading angle
    theta_t_deg = theta_t*180/pi;
    slopet = tan(theta_t); %slope of transverse line to current path
    s=0;m=0; %initialize sums
    x=x2;y=y2; %initialize scan point
    n=-1; %scan along transverse line, to the right of step endpoint
    while ((y<1199)&&(y>2)&&(x<1599)&&(x>2))
        n=n+1;
        x=x2+n*cos(theta_t); y=y2+n*sin(theta_t);
        r = round(1201-y);c=round(x); %convert to row, column indexing
        pix=p1d(r,c); %get pixel value
        s=s+n*pix^2; %sum weighted pixel values
        m=m+pix^2; %sum pixel values
    end
    plot(x,1200-y,'bo')
    n=0; %scan left
    x=x2;y=y2; %initialize scan point
    while ((y<1199)&&(y>2)&&(x<1599)&&(x>2))
        n=n-1;
        x=x2+n*cos(theta_t); y=y2+n*sin(theta_t);
        r = round(1201-y);c=round(x); %convert to row, column indexing
        pix=p1d(r,c); %get pixel value
        s=s+n*pix^2; %sum weighted pixel values
        m=m+pix^2; %sum pixel values
    end
    plot(x,1200-y,'ro')
    com=s/m; %"center of mass" of pixel value along transverse line in units of n
    x3=x2+com*cos(theta_t); y3=y2+com*sin(theta_t); %corrected step end
    theta = atan2((y3-y1),(x3-x1)); %corrected heading angle
    theta_deg = theta*180/pi;
    %lather, rinse, and repeat
    pathx(step)=x3; pathy(step)=y3; %save this step in the path
    plot(x3,1200-y3, 'r*')
    drawnow
    x1=x3; y1=y3; %previous step end becomes new step start
    
end
 
plot(pathx, 1200-pathy)
%axis([0 1200 0 1600])

        
        